package Mythtv::TVTome;

use strict;
use Data::Dumper;
use Carp;
use LWP::Simple;
use HTML::TokeParser;
use Cache::FileCache;

use constant TVTOME => 'http://www.tvtome.com/';

use Parameters ();              # Application wide configuration
use vars qw(%items);            # Grab the datastructure from the config file
*items = \%Parameters::items;   # Alias the variable out


sub new {
    my $pkg = shift;

    my $self = bless { }, $pkg;
    $self->{_cache} = new Cache::FileCache();
    $self->{config}=\%items;

    return $self;
}

{
    sub _findResults {
        my $self = shift;
        my $content = shift;
        my $parser = HTML::TokeParser->new(\$content);
    
        my ($tag, @list);
    
        while ($tag = $parser->get_token()) {
            if ($parser->get_text =~ 'Show search for') {
                while ($tag = $parser->get_tag()){
            
                    if ($tag->[0] eq 'a') {
                        my $startcrap = '<a href="';
                        my $endcrap = '">';
                        $tag->[3] =~ s/$startcrap//g;
                        $tag->[3] =~ s/$endcrap//g;
    
                        push @list, {show => $parser->get_text(), url=>$tag->[3]};
                    }
                    last if ($parser->get_text =~ /Here are some hints/);
    
                }
            }
        }
    
        return \@list;
    
    }
    
    sub _getEPs {
        my $self = shift;
        my $content = shift;
        my $parser = HTML::TokeParser->new(\$content);
    
        my ($tag, @list, @test);
        while ($tag = $parser->get_token()) {
    
            if ($parser->get_text =~ 'Season') {

                while ($tag = $parser->get_tag('tr')){

                    # We know the rows we want are with the align=center tag
                    if ($$tag[3] eq '<tr align="center">') {
                        # Episode number
                        $parser->get_tag('td');
                        my $epno = $parser->get_text;
        
                        $parser->get_tag('td');
                        my $seasonep = $parser->get_text;
        
                        $parser->get_tag('td');
                        my $prod = $parser->get_text;
    
                        $parser->get_tag('td');
                        my $airdate = $parser->get_text;
        
                        $parser->get_tag('td'); # There seems to be a blank space here in the table;
        
                        my $tag = $parser->get_tag('a');
                        my $startcrap = '<a href="';
                        my $endcrap = '">';
                        $tag->[3] =~ s/$startcrap//g;
                        $tag->[3] =~ s/$endcrap//g;
                        my $epurl = $tag->[3];
            
                        my $epname = $parser->get_text;
       
                        push @list, {
                                     epno     => $epno,
                                     seasonep => $seasonep,
                                     prod     => $prod,
                                     airdate  => $airdate,
                                     epname   => $epname,
                                     epurl    => $epurl,
                                    }
                    }
                }
             }
     
        }
    
        return \@list;
    
    }
    
    sub _getSynopsis {
        my $self = shift;
        my $content = shift;
        my $parser = HTML::TokeParser->new(\$content);
    
        my ($tag, $synopsis);
        while ($tag = $parser->get_token()) {
            if ($parser->get_text =~ 'Synopsis') {
                $parser->get_tag('tr');
                $parser->get_tag();
                $synopsis .=$parser->get_text();
                while ($tag->[0] ne 'td') {
                    $tag = $parser->get_tag();
                    my $text = $parser->get_text();
                    $synopsis .= $text if (defined $text && length($text) > 0);
                }
            }
    
        }
        return \$synopsis;
    }

    sub _getFirstAired {
        my $self = shift;
        my $content = shift;
        my $parser = HTML::TokeParser->new(\$content);

        my ($tag, $airdate);
        while ($tag = $parser->get_token()) {
            if ($parser->get_text =~ 'Show Information') {

                while ($tag = $parser->get_token()) {

                    if ($parser->get_text =~ 'First Aired') {
                        $parser->get_tag('td'); 
                        $airdate =$parser->get_text()||'Unavailable';
                    }

                }
            }

        }
        $airdate  =~ s/[^0-9]//g;
        return \$airdate;

    }

    sub _getDirector {
        my $self = shift;
        my $content = shift;
        my $parser = HTML::TokeParser->new(\$content);

        my ($tag, $director);
        while ($tag = $parser->get_token()) {
            if ($parser->get_text =~ 'Show Crew') {
                $parser->get_tag('a');
                $parser->get_tag('a');
                $parser->get_tag('a');
                $director .=$parser->get_text()||'Unavailable';
            }

        }
        return \$director;

    }

    sub _getRunningTime {
        my $self = shift;
        my $content = shift;
        my $parser = HTML::TokeParser->new(\$content);

        my ($tag, $runtime);
        while ($tag = $parser->get_token()) {
            if ($parser->get_text =~ 'Show Information') {

                while ($tag = $parser->get_token()) {

                    if ($parser->get_text =~ 'Running Time') {
                        $parser->get_tag('td');
                        $runtime =$parser->get_text()||'Unavailable';
                    }

                }
                $runtime .=$parser->get_text()||'Unavailable';
            }

        }
        $runtime =~ s/[^0-9]//g;
        return \$runtime;

    }

    
    sub _findSingle {
        my $self = shift;
        my $content = shift;
        my $parser = HTML::TokeParser->new(\$content);
    
        my ($tag, @list);
    
        while ($tag = $parser->get_token()) {
            if ($tag->[0] eq 'S' && $tag->[1] eq 'input') {
                if ($tag->[2]->{name} =~ 'returnTo') {
                    return $tag->[2]->{value};
                }
            }
        }
     
    }

    sub _getContent {
        my $self = shift;
        my $url  = shift;
        my $content = $self->{_cache}->get($url);
        if (! defined $content) {
            $content = get($url);
            $self->{_cache}->set($url, $content, $self->{config}{tvtome}{cacheexpy});
        }

        return $content;

    }

}

sub find {
    my $self = shift;
    my %args = @_;

    croak "Must supply show!" unless (defined $args{show}); 

    $args{show}=~s/\s/%20/g;

    my $url = $self->{config}{tvtome}{url}.'tvtome/servlet/Search?searchType=show&searchString='.$args{show};

    my $content = $self->_getContent($url);

    # Attempt to autodetect the search function returning with the only series that matches
    # the serarch criteria
    if ($content =~ /Welcome to/i) {
        # there is only one match and we have dragged back the home page
        my $url = $self->_findSingle($content);    
        $args{show} =~ s/%20/ /g;
        return [{show => $args{show}, url => $url}];
    } else {
        return $self->_findResults($content);
    }
}

sub getEPList {
    my $self = shift;
    my %args = @_;

    croak "Must supply url!" unless (defined $args{url}); 

    my $url = $self->{config}{tvtome}{url}.$args{url}.'eplist.html';
    my $content = $self->_getContent($url)  or carp "can't connect to server";

    return $self->_getEPs($content);


}

sub getSynopsis {
    my $self = shift;
    my %args = @_;

    croak "Must supply url!" unless (defined $args{url});

    my $url = $self->{config}{tvtome}{url}.$args{url};
    my $content = $self->_getContent($url)  or carp "can't connect to server";

    return $self->_getSynopsis($content);

}

sub getFirstAirDate {
    my $self = shift;
    my %args = @_;

    croak "Must supply url!" unless (defined $args{url});

    my $url = $self->{config}{tvtome}{url}.$args{url};
    my $content = $self->_getContent($url)  or carp "can't connect to server";

    return $self->_getFirstAired($content);

}

sub getDirector {
    my $self = shift;
    my %args = @_;

    croak "Must supply url!" unless (defined $args{url});

    my $url = $self->{config}{tvtome}{url}.$args{url};
    my $content = $self->_getContent($url)  or carp "can't connect to server";

    return $self->_getDirector($content);

}

sub getRunningTime {
    my $self = shift;
    my %args = @_;

    croak "Must supply url!" unless (defined $args{url});

    my $url = $self->{config}{tvtome}{url}.$args{url};
    my $content = $self->_getContent($url)  or carp "can't connect to server";

    return $self->_getRunningTime($content);

}


1
