#!/usr/bin/perl -w

use strict;
use Mythtv::VideoMetaData;
use Mythtv::TVTome;

use Data::Dumper;

my $vmd  = Mythtv::VideoMetaData->new();
my $tvto = Mythtv::TVTome->new();

my $noplot = $vmd->getNoPlot();

my $dbh = DBI->connect("DBI:mysql:mythconverg:myth1", 'root', undef);
my $sql = "update videometadata set plot = ?, userrating = 0, title = ? where intid = ?";

my $mythshow;
foreach my $thisshow (@$noplot) {

    system("clear");
    print "# ---------------------------------- #\n";
    print "    TVTome updater for Mythvideo\n";
    print "# ---------------------------------- #\n";
    print "Press 'enter' to move to the next video\n";
    print "Enter 'skip' to skip over a remembered series\n\n";
    # Make sense of arbitrary filenames here
    my $inmythshow;
    print "Processing filename:  \"$thisshow->{filename}\"\n";
    print "Enter the show name for \"$thisshow->{title}\": ";
    print "[$mythshow] " if (defined $mythshow);

 
    $inmythshow=<STDIN>; 
    chomp($inmythshow);
    exit if ($inmythshow =~ /q|quit|exit/i);

    # Remember the previous choice
    $inmythshow = $mythshow if ($inmythshow eq '');

    if (!defined $inmythshow || $inmythshow eq 'skip' || ($inmythshow eq '' && $mythshow eq '' ) ) {
       
        $mythshow = $inmythshow;
        next;
    } else {
        # Get the new choice.
        $mythshow = $inmythshow;
    }
   
    print "Enter the Season: ";
    my $mythseason= <STDIN>;
    print "Enter the Episode: ";
    my $mythepisode= <STDIN>;

    chomp($mythseason);
    chomp($mythepisode);

    # Go checkout TVTome now
    print "Searching for \"$mythshow\"..\n";
    my $search = $tvto->find(show => $mythshow);

    # If there is only one match, we dont bother displaying
    # a list.
    my ($seriesInput, $show);
    if (scalar(@$search) > 1) {

        print "Your search produced the following results:\n";
        my $i=0;

        foreach (@$search) {
            ++$i;
            print "$i. $$_{show}\n";
        }
        print "--------------\n";
        print "Enter your choice: ";
        $seriesInput = <STDIN>;
        die "Must enter a number" unless ($seriesInput =~ /\d/);
        die "Invalid choice." unless ($seriesInput < scalar(@$search)+1 );
    
        $show = $$search[($seriesInput-1)]->{show};
        print 'You have chosen: '.$show."\nSearching....\n\n";
    } else {
        $seriesInput = 1;
        $show = $$search[($seriesInput-1)]->{show};
    } 

    # Go and get all the episodes for the series
    my $epsearch = $tvto->getEPList(url => $$search[($seriesInput-1)]->{url});
    
    #print "Select the episode that matches $show - S${mythseason}E${mythepisode}:\n ";

    my $i=0;
    foreach (@$epsearch) {
        ++$i;
        print "$i.\t$$_{seasonep}\t$$_{airdate}\t$$_{epname}\n";
    }

    print "--------------------------------\n";
    print "Finished episode search for: $show - S${mythseason}E${mythepisode}.\n";
    print "Enter your choice: ";
    my $epInput = <STDIN>;
    chomp($epInput);

    die "Must enter a number" unless ($epInput =~ /\d/);
    die "Invalid choice." unless ($epInput < scalar(@$epsearch)+1 );

    my $episode = $$epsearch[($epInput-1)]->{epname};
    print 'You have chosen episode: '.$episode."\nSearching....\n\n";

    my $synopsis =  $tvto->getSynopsis(url => $$epsearch[($epInput-1)]->{epurl});
    my $director =  $tvto->getDirector(url =>  $$search[($seriesInput-1)]->{url});
    my $runningTime  =  $tvto->getRunningTime(url =>  $$search[($seriesInput-1)]->{url});
    my $firstAirDate =  $tvto->getFirstAirDate(url => $$search[($seriesInput-1)]->{url});
    print "Synopsis for $show, \"$episode\":\n";
    print $$synopsis;
    print "\n";

    print "Length of synopsis: ".length($$synopsis)."\n";
    print "\nSynopsis doesnt seem to be available!\n" unless (length($$synopsis) > 24);
    $$synopsis = "None" unless (length($$synopsis) > 24);

    print "--------------------------------\n";
    my $title = $mythshow.' - S'.sprintf("%02s",$mythseason).'E'.sprintf("%02s",$mythepisode).' - '.$$epsearch[($epInput-1)]->{epname};
    print "Title:        $title\n";
    print "Diretor:      $$director\n";
    print "Running Time: $$runningTime\n";
    print "Aired:        ".substr($$epsearch[($epInput-1)]->{airdate}, -4)."\n";
    print "Mythtv ID:    ".$thisshow->{intid}."\n";
    print "TVTome:       ". $$search[($seriesInput-1)]->{show}."\n";
    print "--------------------------------\n";

    print "\nInsert this into mythconverg.videometadata? [Y|n]";
    my $addInput = <STDIN>;
    chomp ($addInput);

    if ($addInput !~ /^n/i) {
         print "\n Add synopsis for $show, \"$episode\":\n";
         $vmd->updateVideo(plot     => $$synopsis,
                           title    => $title,
                           director => $$director,
                           year     => $$firstAirDate,
                           length   => $$runningTime,
                           intid    => $thisshow->{intid});
         
    } else {
        print "Skipping synopsis for $show, \"$episode\":\n";
    }



}
